//+------------------------------------------------------------------+
//|                                              Check_TP_and_SL.mq5 |
//|                        Copyright 2016, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property description "   StopLoss  TakeProfit."
#property description "     MQL4  MQL5  ."
//---     MQL4   
#ifdef __MQL5__ 
#define Ask    SymbolInfoDouble(_Symbol,SYMBOL_ASK)
#define Bid    SymbolInfoDouble(_Symbol,SYMBOL_BID)
#endif 
//---     MQL5  MQL4   
#ifdef __MQL4__ 
#define ORDER_TYPE_BUY        OP_BUY
#define ORDER_TYPE_SELL       OP_SELL
#define ORDER_TYPE_BUY_LIMIT  OP_BUYLIMIT
#define ORDER_TYPE_SELL_LIMIT OP_SELLLIMIT
#define ORDER_TYPE_BUY_STOP   OP_BUYSTOP
#define ORDER_TYPE_SELL_STOP  OP_SELLSTOP
#endif 

#property strict
//--- 
double spread;
//+------------------------------------------------------------------+
//|    StopLoss  TakeProfit                  |
//+------------------------------------------------------------------+
bool CheckStopLoss_Takeprofit(ENUM_ORDER_TYPE type,double price,double SL,double TP)
  {
//---   SYMBOL_TRADE_STOPS_LEVEL
   int stops_level=(int)SymbolInfoInteger(_Symbol,SYMBOL_TRADE_STOPS_LEVEL);
   if(stops_level!=0)
     {
      PrintFormat("SYMBOL_TRADE_STOPS_LEVEL=%d: StopLoss  TakeProfit  "+
                  "   %d    ",stops_level,stops_level);
     }
//---
   bool SL_check=false,TP_check=false;
//---   
   switch(type)
     {
      //---   Buy
      case  ORDER_TYPE_BUY:
        {
         //---  StopLoss
         SL_check=(Bid-SL>stops_level*_Point);
         if(!SL_check)
            PrintFormat("  %s   StopLoss=%.5f    %.5f"+
                        " (Bid=%.5f - SYMBOL_TRADE_STOPS_LEVEL=%d )",
                        EnumToString(type),SL,Bid-stops_level*_Point,Bid,stops_level);
         //---  TakeProfit
         TP_check=(TP-Bid>stops_level*_Point);
         if(!TP_check)
            PrintFormat("  %s   TakeProfit=%.5f    %.5f"+
                        " (Bid=%.5f + SYMBOL_TRADE_STOPS_LEVEL=%d )",
                        EnumToString(type),TP,Bid+stops_level*_Point,Bid,stops_level);
         //---   
         return(SL_check&&TP_check);
        }
      //---   Sell
      case  ORDER_TYPE_SELL:
        {
         //---  StopLoss
         SL_check=(SL-Ask>stops_level*_Point);
         if(!SL_check)
            PrintFormat("  %s   StopLoss=%.5f    %.5f "+
                        " (Ask=%.5f + SYMBOL_TRADE_STOPS_LEVEL=%d )",
                        EnumToString(type),SL,Ask+stops_level*_Point,Ask,stops_level);
         //---  TakeProfit
         TP_check=(Ask-TP>stops_level*_Point);
         if(!TP_check)
            PrintFormat("  %s   TakeProfit=%.5f    %.5f "+
                        " (Ask=%.5f - SYMBOL_TRADE_STOPS_LEVEL=%d )",
                        EnumToString(type),TP,Ask-stops_level*_Point,Ask,stops_level);
         //---   
         return(TP_check&&SL_check);
        }
      break;
      //---   BuyLimit
      case  ORDER_TYPE_BUY_LIMIT:
        {
         //---  StopLoss
         SL_check=((price-SL)>stops_level*_Point);
         if(!SL_check)
            PrintFormat("  %s   StopLoss=%.5f    %.5f"+
                        " (Open-StopLoss=%d  ==> SYMBOL_TRADE_STOPS_LEVEL=%d )",
                        EnumToString(type),SL,price-stops_level*_Point,(int)((price-SL)/_Point),stops_level);
         //---  TakeProfit
         TP_check=((TP-price)>stops_level*_Point);
         if(!TP_check)
            PrintFormat("  %s   TakeProfit=%.5f    %.5f"+
                        " (TakeProfit-Open=%d  ==> SYMBOL_TRADE_STOPS_LEVEL=%d )",
                        EnumToString(type),TP,price+stops_level*_Point,(int)((TP-price)/_Point),stops_level);
         //---   
         return(SL_check&&TP_check);
        }
      //---   SellLimit
      case  ORDER_TYPE_SELL_LIMIT:
        {
         //---  StopLoss
         SL_check=((SL-price)>stops_level*_Point);
         if(!SL_check)
            PrintFormat("  %s   StopLoss=%.5f    %.5f "+
                        " (StopLoss-Open=%d  ==> SYMBOL_TRADE_STOPS_LEVEL=%d )",
                        EnumToString(type),SL,price+stops_level*_Point,(int)((SL-price)/_Point),stops_level);
         //---  TakeProfit
         TP_check=((price-TP)>stops_level*_Point);
         if(!TP_check)
            PrintFormat("  %s   TakeProfit=%.5f    %.5f "+
                        " (Open-TakeProfit=%d  ==> SYMBOL_TRADE_STOPS_LEVEL=%d )",
                        EnumToString(type),TP,price-stops_level*_Point,(int)((price-TP)/_Point),stops_level);
         //---   
         return(TP_check&&SL_check);
        }
      break;
      //---   BuyStop
      case  ORDER_TYPE_BUY_STOP:
        {
         //---  StopLoss
         SL_check=((price-SL)>stops_level*_Point);
         if(!SL_check)
            PrintFormat("  %s   StopLoss=%.5f    %.5f"+
                        " (Open-StopLoss=%d  ==> SYMBOL_TRADE_STOPS_LEVEL=%d )",
                        EnumToString(type),SL,price-stops_level*_Point,(int)((price-SL)/_Point),stops_level);
         //---  TakeProfit
         TP_check=((TP-price)>stops_level*_Point);
         if(!TP_check)
            PrintFormat("  %s   TakeProfit=%.5f    %.5f"+
                        " (TakeProfit-Open=%d  ==> SYMBOL_TRADE_STOPS_LEVEL=%d )",
                        EnumToString(type),TP,price-stops_level*_Point,(int)((TP-price)/_Point),stops_level);
         //---   
         return(SL_check&&TP_check);
        }
      //---   SellStop
      case  ORDER_TYPE_SELL_STOP:
        {
         //---  StopLoss
         SL_check=((SL-price)>stops_level*_Point);
         if(!SL_check)
            PrintFormat("  %s   StopLoss=%.5f    %.5f "+
                        " (StopLoss-Open=%d  ==> SYMBOL_TRADE_STOPS_LEVEL=%d )",
                        EnumToString(type),SL,price+stops_level*_Point,(int)((SL-price)/_Point),stops_level);
         //---  TakeProfit
         TP_check=((price-TP)>stops_level*_Point);
         if(!TP_check)
            PrintFormat("  %s   TakeProfit=%.5f    %.5f "+
                        " (Open-TakeProfit=%d  ==> SYMBOL_TRADE_STOPS_LEVEL=%d )",
                        EnumToString(type),TP,price-stops_level*_Point,(int)((price-TP)/_Point),stops_level);
         //---   
         return(TP_check&&SL_check);
        }
      break;
     }

//---
   return false;
  }
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
//---  
   spread=Ask-Bid;
//---     
   int oper=(int)(GetTickCount()%2); //        0  1
   switch(oper)
     {
      //--- 
      case  0:
        {
         //---        TP/SL
         double price=Ask;
         double SL=NormalizeDouble(Bid+2*_Point,_Digits);
         double TP=NormalizeDouble(Bid-2*_Point,_Digits);
         //---  
         PrintFormat("Buy at %.5f   SL=%.5f   TP=%.5f  Bid=%.5f",price,SL,TP,Bid);
         if(!CheckStopLoss_Takeprofit(ORDER_TYPE_BUY,price,SL,TP))
            Print(" StopLoss  TakeProfit  !");
         //---    ,    
         Buy(price,SL,TP);
         Print("  -----------------");
        }
      break;
      //--- 
      case  1:
        {
         //---        TP/SL
         double price=Bid;
         double SL=NormalizeDouble(Ask-2*_Point,_Digits);
         double TP=NormalizeDouble(Ask+2*_Point,_Digits);
         //---  
         PrintFormat("Sell at %.5f   SL=%.5f   TP=%.5f  Ask=%.5f",price,SL,TP,Ask);
         if(!CheckStopLoss_Takeprofit(ORDER_TYPE_SELL,price,SL,TP))
            Print(" StopLoss  TakeProfit  !");
         //---    ,    
         Sell(price,SL,TP);
         Print("  -----------------");
        }
      break;
      //---
     }
//---     
   oper=(int)(GetTickCount()%4); //     4    0  3
   switch(oper)
     {
      //---  BuyLimit
      case  0:
        {
         //---        TP/SL
         double price=NormalizeDouble(Ask-200*_Point,_Digits);
         double SL=NormalizeDouble(price-2*_Point,_Digits);
         double TP=NormalizeDouble(price+2*_Point,_Digits);
         //---  
         PrintFormat("BuyLimt at %.5f   SL=%.5f   TP=%.5f",price,SL,TP);
         if(!CheckStopLoss_Takeprofit(ORDER_TYPE_BUY_LIMIT,price,SL,TP))
            Print(" StopLoss  TakeProfit  !");
         //---    ,    
         BuyLimit(price,SL,TP);
         Print("  -----------------");
        }
      break;
      //---  BuyStop
      case  1:
        {
         //---        TP/SL
         double price=NormalizeDouble(Ask+200*_Point,_Digits);
         double SL=NormalizeDouble(price-2*_Point,_Digits);
         double TP=NormalizeDouble(price+2*_Point,_Digits);
         //---  
         PrintFormat("BuyStop at %.5f   SL=%.5f  TP=%.5f",price,SL,TP);
         if(!CheckStopLoss_Takeprofit(ORDER_TYPE_BUY_STOP,price,SL,TP))
            Print(" StopLoss  TakeProfit  !");
         //---     ,    
         BuyStop(price,SL,TP);
         Print("  -----------------");
        }
      break;
      //---  SellLimit
      case  2:
        {
         //---        TP/SL
         double price=NormalizeDouble(Bid+200*_Point,_Digits);
         double SL=NormalizeDouble(price+2*_Point,_Digits);
         double TP=NormalizeDouble(price-2*_Point,_Digits);
         //---  
         PrintFormat("SellLimit at %.5f   SL=%.5f  TP=%.5f",price,SL,TP);
         if(!CheckStopLoss_Takeprofit(ORDER_TYPE_SELL_LIMIT,price,SL,TP))
            Print(" StopLoss  TakeProfit  !");
         //---     ,    
         SellLimit(price,SL,TP);
         Print("  -----------------");
        }
      break;
      //---  SellStop
      case  3:
        {
         //---        TP/SL
         double price=NormalizeDouble(Bid-200*_Point,_Digits);
         double SL=NormalizeDouble(price+2*_Point,_Digits);
         double TP=NormalizeDouble(price-2*_Point,_Digits);
         //---  
         PrintFormat("SellStop at %.5f  SL=%.5f  TP=%.5f ",price,SL,TP);
         if(!CheckStopLoss_Takeprofit(ORDER_TYPE_SELL_STOP,price,SL,TP))
            Print(" StopLoss  TakeProfit  !");
         //---     ,    
         SellStop(price,SL,TP);
         Print("  -----------------");
        }
      break;
     }
//---
  }
//+------------------------------------------------------------------+
//|                                                  |
//+------------------------------------------------------------------+
bool Buy(double price,double sl,double tp)
  {
//---   MQL4
#ifdef __MQL4__
   int ticket=OrderSend(Symbol(),OP_BUY,1,price,10,sl,tp);
   if(ticket<0)
      PrintFormat("OrderSend error %d",GetLastError());
#endif
//---   MQL5         
#ifdef __MQL5__
//---      
   MqlTradeRequest request={0};
   MqlTradeResult  result={0};
//---  
   request.action   =TRADE_ACTION_DEAL;                     //   
   request.symbol   =Symbol();                              // 
   request.volume   =1;                                     //   1 
   request.type     =ORDER_TYPE_BUY;                        //  
   request.price    =price;                                 //   
   request.sl       =sl;                                    //  StopLoss
   request.tp       =tp;                                    //  TakeProfit
   request.deviation=10;                                    //    
   request.magic    =123456;                                // MagicNumber 
//---  
   if(!OrderSend(request,result))
      PrintFormat("OrderSend error %d",GetLastError());     //     ,   
//---   
   PrintFormat("retcode=%u  deal=%I64u  order=%I64u",result.retcode,result.deal,result.order);
#endif  
//---
   return(false);
  }
//+------------------------------------------------------------------+
//|                                                  |
//+------------------------------------------------------------------+
bool Sell(double price,double sl,double tp)
  {
//---   MQL4
#ifdef __MQL4__
   int ticket=OrderSend(Symbol(),OP_SELL,1,price,10,sl,tp);
   if(ticket<0)
      PrintFormat("OrderSend error %d",GetLastError());
#endif
//---   MQL5         
#ifdef __MQL5__
//---      
   MqlTradeRequest request={0};
   MqlTradeResult  result={0};
//---  
   request.action   =TRADE_ACTION_DEAL;                     //   
   request.symbol   =Symbol();                              // 
   request.volume   =1;                                     //   0.1 
   request.type     =ORDER_TYPE_SELL;                       //  
   request.price    =price;                                 //   
   request.sl       =sl;                                    //  StopLoss
   request.tp       =tp;                                    //  TakeProfit
   request.deviation=10;                                    //    
   request.magic    =123456;                                // MagicNumber 
//---  
   if(!OrderSend(request,result))
      PrintFormat("OrderSend error %d",GetLastError());     //     ,   
//---   
   PrintFormat("retcode=%u  deal=%I64u  order=%I64u",result.retcode,result.deal,result.order);
#endif  
//---
   return(false);
  }
//+------------------------------------------------------------------+
//|     Buy Limit                           |
//+------------------------------------------------------------------+
bool BuyLimit(double price,double sl,double tp)
  {
//---   MQL4
#ifdef __MQL4__
   int ticket=OrderSend(Symbol(),OP_BUYLIMIT,1,price,10,sl,tp);
   if(ticket<0)
      PrintFormat("OrderSend error %d",GetLastError());
#endif
//---   MQL5         
#ifdef __MQL5__
//---      
   MqlTradeRequest request={0};
   MqlTradeResult  result={0};
//---  
   request.action   =TRADE_ACTION_PENDING;                  //   
   request.symbol   =Symbol();                              // 
   request.volume   =1;                                     //   1 
   request.type     =ORDER_TYPE_BUY_LIMIT;                  //  
   request.price    =price;                                 //   
   request.sl       =sl;                                    //  StopLoss
   request.tp       =tp;                                    //  TakeProfit
   request.deviation=10;                                    //    
   request.magic    =123456;                                // MagicNumber 
//---  
   if(!OrderSend(request,result))
      PrintFormat("OrderSend error %d",GetLastError());     //     ,   
//---   
   PrintFormat("retcode=%u  deal=%I64u  order=%I64u",result.retcode,result.deal,result.order);
#endif  
//---
   return(false);
  }
//+------------------------------------------------------------------+
//|     Buy Stop                            |
//+------------------------------------------------------------------+
bool BuyStop(double price,double sl,double tp)
  {
//---   MQL4
#ifdef __MQL4__
   int ticket=OrderSend(Symbol(),OP_BUYSTOP,1,price,10,sl,tp);
   if(ticket<0)
      PrintFormat("OrderSend error %d",GetLastError());
#endif
//---   MQL5         
#ifdef __MQL5__
//---      
   MqlTradeRequest request={0};
   MqlTradeResult  result={0};
//---  
   request.action   =TRADE_ACTION_PENDING;                  //   
   request.symbol   =Symbol();                              // 
   request.volume   =1;                                     //   1 
   request.type     =ORDER_TYPE_BUY_STOP;                   //  
   request.price    =price;                                 //   
   request.sl       =sl;                                    //  StopLoss
   request.tp       =tp;                                    //  TakeProfit
   request.deviation=10;                                    //    
   request.magic    =123456;                                // MagicNumber 
//---  
   if(!OrderSend(request,result))
      PrintFormat("OrderSend error %d",GetLastError());     //     ,   
//---   
   PrintFormat("retcode=%u  deal=%I64u  order=%I64u",result.retcode,result.deal,result.order);
#endif  
//---
   return(false);
  }
//+------------------------------------------------------------------+
//|     Buy Limit                           |
//+------------------------------------------------------------------+
bool SellLimit(double price,double sl,double tp)
  {
//---   MQL4
#ifdef __MQL4__
   int ticket=OrderSend(Symbol(),OP_SELLLIMIT,1,price,10,sl,tp);
   if(ticket<0)
      PrintFormat("OrderSend error %d",GetLastError());
#endif
//---   MQL5         
#ifdef __MQL5__
//---      
   MqlTradeRequest request={0};
   MqlTradeResult  result={0};
//---  
   request.action   =TRADE_ACTION_PENDING;                  //   
   request.symbol   =Symbol();                              // 
   request.volume   =1;                                     //   1 
   request.type     =ORDER_TYPE_SELL_LIMIT;                 //  
   request.price    =price;                                 //   
   request.sl       =sl;                                    //  StopLoss
   request.tp       =tp;                                    //  TakeProfit
   request.deviation=10;                                    //    
   request.magic    =123456;                                // MagicNumber 
//---  
   if(!OrderSend(request,result))
      PrintFormat("OrderSend error %d",GetLastError());     //     ,   
//---   
   PrintFormat("retcode=%u  deal=%I64u  order=%I64u",result.retcode,result.deal,result.order);
#endif  
//---
   return(false);
  }
//+------------------------------------------------------------------+
//|     Buy Stop                            |
//+------------------------------------------------------------------+
bool SellStop(double price,double sl,double tp)
  {
//---   MQL4
#ifdef __MQL4__
   int ticket=OrderSend(Symbol(),OP_SELLSTOP,1,price,10,sl,tp);
   if(ticket<0)
      PrintFormat("OrderSend error %d",GetLastError());
#endif
//---   MQL5         
#ifdef __MQL5__
//---      
   MqlTradeRequest request={0};
   MqlTradeResult  result={0};
//---  
   request.action   =TRADE_ACTION_PENDING;                  //   
   request.symbol   =Symbol();                              // 
   request.volume   =1;                                     //   1 
   request.type     =ORDER_TYPE_SELL_STOP;                  //  
   request.price    =price;                                 //   
   request.sl       =sl;                                    //  StopLoss
   request.tp       =tp;                                    //  TakeProfit
   request.deviation=10;                                    //    
   request.magic    =123456;                                // MagicNumber 
//---  
   if(!OrderSend(request,result))
      PrintFormat("OrderSend error %d",GetLastError());     //     ,   
//---   
   PrintFormat("retcode=%u  deal=%I64u  order=%I64u",result.retcode,result.deal,result.order);
#endif  
//---
   return(false);
  }
//+------------------------------------------------------------------+
